@@ -1,18 +1,16 @@
-{-# LANGUAGE DataKinds #-}
-{-# LANGUAGE DeriveGeneric #-}
-{-# LANGUAGE InstanceSigs #-}
-{-# LANGUAGE OverloadedStrings #-}
 {-# LANGUAGE QuasiQuotes #-}
-{-# LANGUAGE RecordWildCards #-}
-{-# LANGUAGE TypeApplications #-}
-{-# LANGUAGE TypeFamilies #-}
 
+-- | [Google build oauth2 web server application](https://developers.google.com/identity/protocols/oauth2/web-server)
 module Network.OAuth2.Provider.Google where
 
+import Control.Monad.IO.Class (MonadIO (..))
+import Control.Monad.Trans.Except (ExceptT (..))
 import Crypto.PubKey.RSA.Types
 import Data.Aeson
 import Data.Aeson qualified as Aeson
 import Data.Bifunctor
+import Data.ByteString.Contrib
+import Data.ByteString.Lazy.Char8 qualified as BSL
 import Data.Map.Strict qualified as Map
 import Data.Maybe
 import Data.Set qualified as Set
@@ -24,9 +22,10 @@ import GHC.Generics
 import Jose.Jwa
 import Jose.Jws
 import Jose.Jwt
-import Network.OAuth.OAuth2
+import Network.HTTP.Conduit (Manager)
+import Network.OAuth2
 import Network.OAuth2.Experiment
-import Network.OAuth2.Provider.Utils
+import Network.OAuth2.Provider
 import OpenSSL.EVP.PKey (toKeyPair)
 import OpenSSL.PEM (
   PemPasswordSupply (PwNone),
@@ -39,32 +38,30 @@ import URI.ByteString.QQ
 To test at google playground, set redirect uri to "https://developers.google.com/oauthplayground"
 -}
 
-data Google = Google deriving (Eq, Show)
-
-type instance IdpUserInfo Google = GoogleUser
-
 -- * Authorization Code flow
 
-defaultGoogleApp :: IdpApplication 'AuthorizationCode Google
-defaultGoogleApp =
-  AuthorizationCodeIdpApplication
-    { idpAppClientId = ""
-    , idpAppClientSecret = ""
-    , idpAppScope =
-        Set.fromList
-          [ "https://www.googleapis.com/auth/userinfo.email"
-          , "https://www.googleapis.com/auth/userinfo.profile"
-          ]
-    , idpAppAuthorizeState = "CHANGE_ME"
-    , idpAppAuthorizeExtraParams = Map.empty
-    , idpAppRedirectUri = [uri|http://localhost|]
-    , idpAppName = "default-google-App"
-    , idpAppTokenRequestAuthenticationMethod = ClientSecretBasic
-    , idp = defaultGoogleIdp
+sampleGoogleAuthorizationCodeApp :: AuthorizationCodeApplication
+sampleGoogleAuthorizationCodeApp =
+  AuthorizationCodeApplication
+    { acName = "sample-google-authorization-code-app"
+    , acClientId = ""
+    , acClientSecret = ""
+    , acScope = Set.fromList ["https://www.googleapis.com/auth/userinfo.email", "https://www.googleapis.com/auth/userinfo.profile"]
+    , acAuthorizeState = "CHANGE_ME"
+    , acRedirectUri = [uri|http://localhost|]
+    , acAuthorizeRequestExtraParams = Map.empty
+    , acClientAuthenticationMethod = ClientSecretBasic
     }
 
 -- * Service Account
 
+sampleServiceAccountApp :: Jwt -> JwtBearerApplication
+sampleServiceAccountApp jwt =
+  JwtBearerApplication
+    { jbName = "sample-google-service-account-app"
+    , jbJwtAssertion = unJwt jwt
+    }
+
 -- | Service account key (in JSON format) that download from google
 data GoogleServiceAccountKey = GoogleServiceAccountKey
   { privateKey :: String
@@ -138,23 +135,23 @@ readPemRsaKey pemStr = do
           }
     Nothing -> Left "unable to parse PEM to RSA key"
 
-defaultServiceAccountApp :: Jwt -> IdpApplication 'JwtBearer Google
-defaultServiceAccountApp jwt =
-  JwtBearerIdpApplication
-    { idpAppName = "google-sa-app"
-    , idpAppJwt = unJwt jwt
-    , idp = defaultGoogleIdp
-    }
-
 -- * IDP
 
+fetchUserInfo ::
+  (MonadIO m, HasUserInfoRequest a, FromJSON b) =>
+  IdpApplication i a ->
+  Manager ->
+  AccessToken ->
+  ExceptT BSL.ByteString m b
+fetchUserInfo = conduitUserInfoRequest
+
 defaultGoogleIdp :: Idp Google
 defaultGoogleIdp =
   Idp
-    { idpFetchUserInfo = authGetJSON @(IdpUserInfo Google)
-    , idpAuthorizeEndpoint = [uri|https://accounts.google.com/o/oauth2/v2/auth|]
+    { idpAuthorizeEndpoint = [uri|https://accounts.google.com/o/oauth2/v2/auth|]
     , idpTokenEndpoint = [uri|https://oauth2.googleapis.com/token|]
     , idpUserInfoEndpoint = [uri|https://www.googleapis.com/oauth2/v2/userinfo|]
+    , idpDeviceAuthorizationEndpoint = Just [uri|https://oauth2.googleapis.com/device/code|]
     }
 
 -- requires scope "https://www.googleapis.com/auth/userinfo.profile" to obtain "name".
